<?php

namespace Drupal\group_behavior;

use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\Core\Form\FormStateInterface;

class GroupBehaviorFormHooks {

  public static function alterGroupContentTypeEditForm(&$form, FormStateInterface $form_state, $form_id) {
    /** @var \Drupal\Core\Entity\EntityFormInterface $formObject */
    $formObject = $form_state->getFormObject();
    /** @var \Drupal\group\Entity\GroupContentTypeInterface $config */
    $config = $formObject->getEntity();

    $form['third_party_settings']['#tree'] = TRUE;
    $tpsForm =&$form['third_party_settings']['group_behavior'];
    $tpsForm['#type'] = 'fieldset';
    $tpsForm['#title'] = t('Group Behavior');
    $tpsForm['#description'] = t('Cares about creating, deleting and enriching entities with a shadow group.');

    $tpsForm['enabled'] = [
      '#type' => 'checkbox',
      '#title' => t('Enable group behavior'),
      '#description' => t('If a content of this type is created, updated, deleted, a corresponding group is created, updated, deleted.'),
      '#default_value' => $config->getThirdPartySetting('group_behavior', 'enabled'),
    ];

    // Care that our settings go the right way.
    // This does currently NOT work as the submit does not call the base class.
    // $form['#entity_builders'][] = [static::class, 'groupContentTypeBuilder'];
    // @see \Drupal\group\Entity\Form\GroupContentTypeForm::submitForm
    // @todo Fix this upstream someday.
    array_unshift($form['actions']['submit']['#submit'], [static::class, 'submitGroupContentTypeEditForm']);
  }

  public static function submitGroupContentTypeEditForm(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\Core\Entity\EntityFormInterface $formObject */
    $formObject = $form_state->getFormObject();
    /** @var \Drupal\group\Entity\GroupContentTypeInterface $config */
    $config = $formObject->getEntity();

    static::groupContentTypeBuilder($config->getEntityTypeId(), $config, $form, $form_state);

    $form_state->unsetValue(['third_party_settings', 'group_behavior']);
    if (!$form_state->getValue('third_party_settings')) {
      $form_state->unsetValue('third_party_settings');
    }
  }

  /**
   * TPS group content type entity builder.
   */
  public static function groupContentTypeBuilder($entity_type, ConfigEntityInterface $type, &$form, FormStateInterface $form_state) {
    $tps = $form_state->getValue(['third_party_settings', 'group_behavior']);
    foreach ($tps as $key => $value) {
      // Set all values so we can rely on them in EntityQuery.
      $type->setThirdPartySetting('group_behavior', $key, $value);
    }
  }

}
